@@ -20,9 +20,15 @@ import statistics
 try:
     from asolytics.similar import App, parser_similar
     from asolytics.local import Localization_of_naming
+    from asolytics.reviews import App_reviews, Featured_reviews
+    from asolytics.tags import Tag, Google_play_tags, App_tags
+    from asolytics.tracker import Tracker_google_play
 except:
     from similar import App, parser_similar
     from local import Localization_of_naming
+    from reviews import App_reviews, Featured_reviews
+    from tags import Tag, Google_play_tags, App_tags
+    from tracker import Tracker_google_play
 
 parser = argparse.ArgumentParser()
 parser.add_argument('--key', dest='key', type=str, help='Аналізувати ключову фразу')
@@ -37,6 +43,9 @@ parser.add_argument('--file', dest='file', action='store_true', help='Викор
 parser.add_argument('--extract', dest='extract', type=str, help='Виявити ключові слова які використовуються в метаданих застосунку. Аналізуються заголовок, назва розробника, короткий опис, повний опис, відгуки. (--extract org.telegram.messenger)')
 parser.add_argument('--similar', dest='similar', type=str, help='Аналіз схожих додатків. Використовуйте цей ключ, щоб проаналізувати сторінки ваших конкурентів де ви відображаєтесь в схожих (--similar org.telegram.messenger)')
 parser.add_argument('--local', dest='local', type=str, help='Аналіз локалізації неймінга. Мови на які перекладено сторінку додатку в Goole Play (--local org.thoughtcrime.securesms)')
+parser.add_argument('--reviews', dest='reviews', type=str, help='Аналізувати зафічерені відгуки в різних локалях. Відгуки які знаходяться в топі  (--reviews org.thoughtcrime.securesms)')
+parser.add_argument('--csv', dest='csv', type=str, help='Використовуйте цю опцію, щоб зберегти результат у файл, csv можна відкрити за допомогою електронних таблиць, наприклад Excel (--csv file.csv)')
+parser.add_argument('--tags', dest='tags', type=str, help='Інструмент аналізу тегів в Google Play, перевірка індексації додатку по тегам (--tags org.thoughtcrime.securesms)')
 
 args = parser.parse_args()
 
@@ -49,6 +58,85 @@ options.add_argument("--headless")
 #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 ###################################################################################################################
 
+def tags_analysis(bundleId):
+    print(Fore.GREEN + "* * * Виконую * * *" + Fore.WHITE)
+    print("Bundle ID: " + bundleId)
+    if args.hl != None:
+        hl = args.hl
+        print("Код мови: " + hl)
+    else:
+        hl = "en"
+        print("Код мови: " + hl)
+
+    if args.gl != None:
+        gl = args.gl
+        print("Код країни: " + gl)
+    else:
+        gl = "US"
+        print("Код країни: " + gl)
+
+    gpt = Google_play_tags()
+    tag_labels = gpt.start(bundleId, gl, hl)
+
+    x = PrettyTable()
+    x.field_names = ["Тег", "Тип тегу", "Ключ тегу", "Значущість для кластера", "Кількість додатків", "Індексація по тегу"]
+
+    for tag_label in tag_labels:
+        tag = gpt.find_by_name(tag_label)
+        x.add_row([
+            tag_label,
+            tag.type_tag(),
+            tag.get_keyword(),
+            gpt.average_position(tag_label), 
+            gpt.count_apps(tag_label),
+            gpt.get_index_tag(tag_label)
+        ])
+    x.hrules = ALL
+    x._max_width["Тег"] = 30
+    x._max_width["Ключ тегу"] = 30
+    print(x.get_string(sortby=("Значущість для кластера")))
+
+    if(args.csv != None):
+        save_to_file_csv(x, args.csv)
+
+    print("Всього проаналізовано додатків: {}".format(gpt.count_apps_all))
+    print("* * * Виконано! * * *")
+
+###################################################################################################################
+#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+###################################################################################################################
+
+def reviews_analysis(bundleId):
+    print(Fore.GREEN + "* * * Виконую * * *" + Fore.WHITE)
+    print("Bundle ID: " + bundleId)
+    fr = Featured_reviews()
+    elements = fr.start(bundleId)
+
+    x = PrettyTable()
+    x.field_names = ["Локаль Google Play", "Відгук", "Оцінка"]
+
+    all_rate = []
+    for element in elements:
+        for i, review in enumerate(element.reviews):
+            x.add_row([element.hl, review, element.rates[i]])
+            all_rate.append(element.rates[i])
+    x.hrules = ALL
+    x._max_width["Відгук"] = 100
+    print(x.get_string(sortby=("Локаль Google Play")))
+    if(len(all_rate) > 0):
+        print("Середня оцінка по фічеру відгуків: " + str(round(sum(all_rate) / len(all_rate), 1)))
+
+    if(args.csv != None):
+        save_to_file_csv(x, args.csv)
+
+    print("* * * Виконано! * * *")
+
+###################################################################################################################
+#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+###################################################################################################################
+
 def localization_analysis(bundleId):
     print(Fore.GREEN + "* * * Виконую * * *" + Fore.WHITE)
     print("Bundle ID: " + bundleId)
@@ -65,6 +153,8 @@ def localization_analysis(bundleId):
     list_set = set(names)
     unique_list = (list(list_set))
     print("Кількість використаних мов: " + str(len(unique_list)))
+    if(args.csv != None):
+        save_to_file_csv(x, args.csv)
     print("* * * Виконано! * * *")
     return
 
@@ -115,6 +205,8 @@ def action_parser_similar_app(bundleId):
         print("Середня кількість інсталів додатків на добу, на сторінках яких ви відображаєтесь: від " + str(int(sum(ins_x1_daily) / len(ins_x1_daily))) + " до " + str(int(sum(ins_x2_daily) / len(ins_x2_daily))))
     else:
         print("Ваш додаток ніде не відображається в схожих!")
+    if(args.csv != None):
+        save_to_file_csv(x, args.csv)
     print("* * * Виконано! * * *")
     return
 
@@ -223,6 +315,9 @@ def extract_keywords_metadata_app(bundleId):
     print("Коефіцієнт індексації метаданих (країна - " + gl +"; мова - " + hl +"): "
          + str(int(100 * len(relevant_keys)/len(list_keywords))) + "%")
 
+    if(args.csv != None):
+        save_to_file_csv(x, args.csv)
+
     print("* * * Виконано! * * *")
             
     browser.quit()
@@ -256,23 +351,16 @@ def tracker_position_google_play():
         gl = "US"
         print("Код країни: " + gl)
 
-    #, options=options
-    browser = webdriver.Firefox(service = FirefoxService(GeckoDriverManager().install()), options=options)
-    browser.implicitly_wait(3)
-    browser.maximize_window()
-
-    browser.get("https://play.google.com/store/apps/details?id=" + bundleId + "&hl=" + hl + "&gl=" + gl)
+    tgp = Tracker_google_play()
 
     try :
-        page_app: WebElement = browser.find_element(By.CLASS_NAME, "Fd93Bb")
-        print(Fore.GREEN + "Назва додатка: " + page_app.text + Fore.WHITE)
+        name = tgp.get_name_app(bundleId, hl, gl)
+        print(Fore.GREEN + "Назва додатка: " + name + Fore.WHITE)
     except:
         print("Додаток з таким bundleID ("+ bundleId +") не знайдено в Google Play")
-        browser.quit()
+        tgp.close_browser()
         return
 
-    map_keywords = {}
-
     if(not args.file):
         keywords = args.tracker.split(";")
             
@@ -288,66 +376,64 @@ def tracker_position_google_play():
             print(e.args)
 
     if(len(keywords) == 0):
-        browser.quit()
+        tgp.close_browser()
         return
 
-    for k in keywords:
-        browser.get("https://play.google.com/store/search?q="+ k +"&c=apps&hl=" + hl + "&gl=" + gl)
-        print(k)
-
-        apps = browser.find_elements(By.CLASS_NAME, "Si6A0c")
-
-        links_recommended = []
-
-        try:
-            recommended = browser.find_element(By.CLASS_NAME, "zuJxTd")
-            rapps = recommended.find_elements(By.CLASS_NAME, "Si6A0c")
-
-            for app in rapps:
-                link = app.get_attribute("href")
-                links_recommended.append(link)
-        except Exception as e:
-            print("")
-
-        apps_list = []
-
-        for app in apps:
-            link = app.get_attribute("href")
-            if(not links_recommended.__contains__(link)):
-                try:
-                    app_name = app.find_element(By.CLASS_NAME, "DdYX5").text
-                    company = app.find_element(By.CLASS_NAME, "wMUdtb").text
-                    apps_list.append({"link": link, "app_name": app_name, "company": company})
-                except:
-                    break
-
-        conteins_title = 0
-        conteins_company = 0
-        position = -1
-        for index, app in enumerate(apps_list):
-            my_link = "https://play.google.com/store/apps/details?id=" + bundleId
-            if(app["link"] == my_link):
-                position = index
-            conteins_title += str(app["app_name"]).lower().count(k.lower())
-            conteins_company += str(app["company"]).lower().count(k.lower())
-
-        map_keywords[k] = { 
-            "position": position,
-            "conteins_title": conteins_title,
-            "conteins_company": conteins_company,
-        }        
+    map_keywords = tgp.start(bundleId, gl, hl, keywords)
 
     x = PrettyTable()
-    x.field_names = ["Ключова фраза", "Позиція в пошуку", "Входжень в назви", "Входжень в ім'я розробника"]
+    history = tgp.history(map_keywords)
+    count_days_in_history = tgp.count_day_in_history(history, map_keywords.keys())
+    if count_days_in_history > 0:
+        x.field_names = ["Ключова фраза", "Позиція в пошуку", "Позиція вчора", "Змінилася", "Входжень в назви", "Входжень в ім'я розробника"]
 
-    for item in map_keywords.items():
-        x.add_row([item[0], item[1]["position"] + 1, item[1]["conteins_title"], item[1]["conteins_company"]])
+        for item in map_keywords.items():
+            position_day1 = tgp.find_position_by_day(history, item[0], 1)
+            x.add_row([item[0], 
+                item[1]["position"] + 1,
+                position_day1 + 1, 
+                (item[1]["position"] + 1) - (position_day1 + 1),
+                item[1]["conteins_title"], 
+                item[1]["conteins_company"]])
+    else:
+        x.field_names = ["Ключова фраза", "Позиція в пошуку", "Входжень в назви", "Входжень в ім'я розробника"]
+
+        for item in map_keywords.items():
+            x.add_row([item[0], item[1]["position"] + 1, item[1]["conteins_title"], item[1]["conteins_company"]])
 
     print(x.get_string(sortby=("Позиція в пошуку")))
 
+    x1 = PrettyTable()
+    if count_days_in_history > 0:
+        #x.field_names = ["Ключова фраза", "Позиція в пошуку", "Змінилася", "Входжень в назви", "Входжень в ім'я розробника"]
+        header = ["Ключова фраза"]
+        for i in range(count_days_in_history, 0, -1): header.append("День {}".format(i))
+        header.append("Позиція в пошуку")
+        header.append("Змінилася")
+        header.append("Входжень в назви")
+        header.append("Входжень в ім'я розробника")
+        x1.field_names = header
+
+        for item in map_keywords.items():
+            position_day1 = tgp.find_position_by_day(history, item[0], 1)
+            row = [item[0]]
+            for i in range(count_days_in_history, 0, -1): row.append(tgp.find_position_by_day(history, item[0], i) + 1)
+            row.append(item[1]["position"] + 1)
+            row.append((item[1]["position"] + 1) - (position_day1 + 1))
+            row.append(item[1]["conteins_title"])
+            row.append(item[1]["conteins_company"])
+            x1.add_row(row)
+    else:
+        x1.field_names = ["Ключова фраза", "Позиція в пошуку", "Входжень в назви", "Входжень в ім'я розробника"]
+        for item in map_keywords.items():
+            x1.add_row([item[0], item[1]["position"] + 1, item[1]["conteins_title"], item[1]["conteins_company"]])
+
+    if(args.csv != None):
+        save_to_file_csv(x1, args.csv)
+
     print("* * * Виконано! * * *")
 
-    browser.quit()
+    tgp.close_browser()
     return
 
 ###################################################################################################################
@@ -601,6 +687,8 @@ def trends_google_play(gl, hl):
 
     print(x.get_string(sortby=("Відносна популярність"), reversesort=True)) 
     print("Зібрано ключових слів: " + str(len(map_popularity)))
+    if(args.csv != None):
+        save_to_file_csv(x, args.csv)
     print("* * * Завершено! * * *")
 
     browser.quit()
@@ -612,7 +700,29 @@ def trends_google_play(gl, hl):
 #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 ###################################################################################################################
 
+def save_to_file_csv(x:PrettyTable, path:str):
+    try:
+        with open(path, 'w', newline='', encoding='utf-8') as f_output:
+            f_output.write(x.get_csv_string())
+        f_output.close()
+        print("Збережено в файл: {}".format(os.path.abspath(path)))
+    except Exception as e:
+        print(e.args)
+
+###################################################################################################################
+#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+###################################################################################################################
+
 def main():
+    if(args.tags != None):
+        tags_analysis(args.tags)
+        sys.exit()
+
+    if(args.reviews != None):
+        reviews_analysis(args.reviews)
+        sys.exit()
+
     if(args.local != None):
         localization_analysis(args.local)
         sys.exit()
@@ -743,6 +853,8 @@ def main():
 
     print("Критерій популярності ключа (" + keyword + ") / Кількість похідних саджестів: " + str(len(map)))
 
+    if(args.csv != None):
+        save_to_file_csv(x, args.csv)
     print("* * * Завершено! * * *")
 
     browser.quit()
